==> create-fly-1.21.8-6.0.8-2-sources.jar: com/zurrtum/create/content/fluids/pipes/SmartFluidPipeBlock.java <==
package com.zurrtum.create.content.fluids.pipes;

import com.mojang.serialization.MapCodec;
import com.zurrtum.create.AllBlockEntityTypes;
import com.zurrtum.create.AllShapes;
import com.zurrtum.create.catnip.data.Iterate;
import com.zurrtum.create.catnip.math.VoxelShaper;
import com.zurrtum.create.content.equipment.wrench.IWrenchable;
import com.zurrtum.create.content.fluids.FluidPropagator;
import com.zurrtum.create.foundation.advancement.AdvancementBehaviour;
import com.zurrtum.create.foundation.block.IBE;
import com.zurrtum.create.foundation.block.NeighborUpdateListeningBlock;
import com.zurrtum.create.foundation.block.ProperWaterloggedBlock;
import net.minecraft.class_10;
import net.minecraft.class_10225;
import net.minecraft.class_1309;
import net.minecraft.class_1750;
import net.minecraft.class_1799;
import net.minecraft.class_1922;
import net.minecraft.class_1937;
import net.minecraft.class_1953;
import net.minecraft.class_2248;
import net.minecraft.class_2338;
import net.minecraft.class_2341;
import net.minecraft.class_2350;
import net.minecraft.class_2350.class_2351;
import net.minecraft.class_2591;
import net.minecraft.class_265;
import net.minecraft.class_2680;
import net.minecraft.class_2689.class_2690;
import net.minecraft.class_2738;
import net.minecraft.class_3218;
import net.minecraft.class_3610;
import net.minecraft.class_3726;
import net.minecraft.class_4209;
import net.minecraft.class_4538;
import net.minecraft.class_5819;
import net.minecraft.class_9904;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

public class SmartFluidPipeBlock extends class_2341 implements IBE<SmartFluidPipeBlockEntity>, IAxisPipe, IWrenchable, ProperWaterloggedBlock, NeighborUpdateListeningBlock {

    public static final MapCodec<SmartFluidPipeBlock> CODEC = method_54094(SmartFluidPipeBlock::new);

    public SmartFluidPipeBlock(class_2251 p_i48339_1_) {
        super(p_i48339_1_);
        method_9590(method_9564().method_11657(WATERLOGGED, false));
    }

    @Override
    protected void method_9515(class_2690<class_2248, class_2680> builder) {
        builder.method_11667(field_11007, field_11177, WATERLOGGED);
    }

    @Override
    public class_2680 method_9605(class_1750 ctx) {
        class_2680 stateForPlacement = super.method_9605(ctx);
        class_2351 prefferedAxis = null;
        class_2338 pos = ctx.method_8037();
        class_1937 world = ctx.method_8045();
        for (class_2350 side : Iterate.directions) {
            if (!prefersConnectionTo(world, pos, side))
                continue;
            if (prefferedAxis != null && prefferedAxis != side.method_10166()) {
                prefferedAxis = null;
                break;
            }
            prefferedAxis = side.method_10166();
        }

        if (prefferedAxis == class_2351.field_11052)
            stateForPlacement = stateForPlacement.method_11657(field_11007, class_2738.field_12471).method_11657(field_11177, stateForPlacement.method_11654(field_11177).method_10153());
        else if (prefferedAxis != null) {
            if (stateForPlacement.method_11654(field_11007) == class_2738.field_12471)
                stateForPlacement = stateForPlacement.method_11657(field_11007, class_2738.field_12475);
            for (class_2350 direction : ctx.method_7718()) {
                if (direction.method_10166() != prefferedAxis)
                    continue;
                stateForPlacement = stateForPlacement.method_11657(field_11177, direction.method_10153());
            }
        }

        return withWater(stateForPlacement, ctx);
    }

    protected boolean prefersConnectionTo(class_4538 reader, class_2338 pos, class_2350 facing) {
        class_2338 offset = pos.method_10093(facing);
        class_2680 blockState = reader.method_8320(offset);
        return FluidPipeBlock.canConnectTo(reader, offset, blockState, facing);
    }

    @Override
    public void method_66388(class_2680 state, class_3218 world, class_2338 pos, boolean isMoving) {
        if (!world.field_9236)
            FluidPropagator.propagateChangedPipe(world, pos, state);
    }

    @Override
    public boolean method_9558(class_2680 p_196260_1_, class_4538 p_196260_2_, class_2338 p_196260_3_) {
        return true;
    }

    @Override
    public void method_9615(class_2680 state, class_1937 world, class_2338 pos, class_2680 oldState, boolean isMoving) {
        if (world.field_9236)
            return;
        if (state != oldState)
            world.method_64311(pos, this, 1, class_1953.field_9310);
    }

    @Override
    public void neighborUpdate(class_2680 state, class_1937 world, class_2338 pos, class_2248 otherBlock, class_2338 neighborPos, boolean isMoving) {
        class_2350 d = FluidPropagator.validateNeighbourChange(state, world, pos, otherBlock, neighborPos, isMoving);
        if (d == null)
            return;
        if (!isOpenAt(state, d))
            return;
        world.method_64311(pos, this, 1, class_1953.field_9310);
    }

    @Override
    public void method_9612(
        class_2680 state,
        class_1937 world,
        class_2338 pos,
        class_2248 otherBlock,
        @Nullable class_9904 wireOrientation,
        boolean isMoving
    ) {
        class_4209.method_19472(world, pos);
    }

    public static boolean isOpenAt(class_2680 state, class_2350 d) {
        return d.method_10166() == getPipeAxis(state);
    }

    @Override
    public void method_9588(class_2680 state, class_3218 world, class_2338 pos, class_5819 r) {
        FluidPropagator.propagateChangedPipe(world, pos, state);
    }

    protected static class_2351 getPipeAxis(class_2680 state) {
        return state.method_11654(field_11007) == class_2738.field_12471 ? class_2351.field_11052 : state.method_11654(field_11177).method_10166();
    }

    @Override
    public class_265 method_9530(class_2680 state, class_1922 p_220053_2_, class_2338 p_220053_3_, class_3726 p_220053_4_) {
        class_2738 face = state.method_11654(field_11007);
        VoxelShaper shape = face == class_2738.field_12475 ? AllShapes.SMART_FLUID_PIPE_FLOOR : face == class_2738.field_12473 ? AllShapes.SMART_FLUID_PIPE_CEILING : AllShapes.SMART_FLUID_PIPE_WALL;
        return shape.get(state.method_11654(field_11177));
    }

    @Override
    public void method_9567(class_1937 pLevel, class_2338 pPos, class_2680 pState, class_1309 pPlacer, class_1799 pStack) {
        super.method_9567(pLevel, pPos, pState, pPlacer, pStack);
        AdvancementBehaviour.setPlacedBy(pLevel, pPos, pPlacer);
    }

    @Override
    public class_2351 getAxis(class_2680 state) {
        return getPipeAxis(state);
    }

    @Override
    protected boolean method_9516(class_2680 state, class_10 pathComputationType) {
        return false;
    }

    @Override
    public class_2680 method_9559(
        class_2680 pState,
        class_4538 pLevel,
        class_10225 tickView,
        class_2338 pCurrentPos,
        class_2350 pFacing,
        class_2338 pFacingPos,
        class_2680 pFacingState,
        class_5819 random
    ) {
        updateWater(pLevel, tickView, pState, pCurrentPos);
        return pState;
    }

    @Override
    public class_3610 method_9545(class_2680 pState) {
        return fluidState(pState);
    }

    @Override
    public Class<SmartFluidPipeBlockEntity> getBlockEntityClass() {
        return SmartFluidPipeBlockEntity.class;
    }

    @Override
    public class_2591<? extends SmartFluidPipeBlockEntity> getBlockEntityType() {
        return AllBlockEntityTypes.SMART_FLUID_PIPE;
    }

    @Override
    protected @NotNull MapCodec<? extends class_2341> method_53969() {
        return field_46280;
    }

}

==> create-fly-1.21.10-6.0.8-3-sources.jar: com/zurrtum/create/content/fluids/pipes/SmartFluidPipeBlock.java <==
package com.zurrtum.create.content.fluids.pipes;

import com.mojang.serialization.MapCodec;
import com.zurrtum.create.AllBlockEntityTypes;
import com.zurrtum.create.AllShapes;
import com.zurrtum.create.catnip.data.Iterate;
import com.zurrtum.create.catnip.math.VoxelShaper;
import com.zurrtum.create.content.equipment.wrench.IWrenchable;
import com.zurrtum.create.content.fluids.FluidPropagator;
import com.zurrtum.create.foundation.advancement.AdvancementBehaviour;
import com.zurrtum.create.foundation.block.IBE;
import com.zurrtum.create.foundation.block.NeighborUpdateListeningBlock;
import com.zurrtum.create.foundation.block.ProperWaterloggedBlock;
import net.minecraft.class_10;
import net.minecraft.class_10225;
import net.minecraft.class_1309;
import net.minecraft.class_1750;
import net.minecraft.class_1799;
import net.minecraft.class_1922;
import net.minecraft.class_1937;
import net.minecraft.class_1953;
import net.minecraft.class_2248;
import net.minecraft.class_2338;
import net.minecraft.class_2341;
import net.minecraft.class_2350;
import net.minecraft.class_2350.class_2351;
import net.minecraft.class_2591;
import net.minecraft.class_265;
import net.minecraft.class_2680;
import net.minecraft.class_2689.class_2690;
import net.minecraft.class_2738;
import net.minecraft.class_3218;
import net.minecraft.class_3610;
import net.minecraft.class_3726;
import net.minecraft.class_4538;
import net.minecraft.class_5819;
import net.minecraft.class_9904;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

public class SmartFluidPipeBlock extends class_2341 implements IBE<SmartFluidPipeBlockEntity>, IAxisPipe, IWrenchable, ProperWaterloggedBlock, NeighborUpdateListeningBlock {

    public static final MapCodec<SmartFluidPipeBlock> CODEC = method_54094(SmartFluidPipeBlock::new);

    public SmartFluidPipeBlock(class_2251 p_i48339_1_) {
        super(p_i48339_1_);
        method_9590(method_9564().method_11657(WATERLOGGED, false));
    }

    @Override
    protected void method_9515(class_2690<class_2248, class_2680> builder) {
        builder.method_11667(field_11007, field_11177, WATERLOGGED);
    }

    @Override
    public class_2680 method_9605(class_1750 ctx) {
        class_2680 stateForPlacement = super.method_9605(ctx);
        class_2351 prefferedAxis = null;
        class_2338 pos = ctx.method_8037();
        class_1937 world = ctx.method_8045();
        for (class_2350 side : Iterate.directions) {
            if (!prefersConnectionTo(world, pos, side))
                continue;
            if (prefferedAxis != null && prefferedAxis != side.method_10166()) {
                prefferedAxis = null;
                break;
            }
            prefferedAxis = side.method_10166();
        }

        if (prefferedAxis == class_2351.field_11052)
            stateForPlacement = stateForPlacement.method_11657(field_11007, class_2738.field_12471).method_11657(field_11177, stateForPlacement.method_11654(field_11177).method_10153());
        else if (prefferedAxis != null) {
            if (stateForPlacement.method_11654(field_11007) == class_2738.field_12471)
                stateForPlacement = stateForPlacement.method_11657(field_11007, class_2738.field_12475);
            for (class_2350 direction : ctx.method_7718()) {
                if (direction.method_10166() != prefferedAxis)
                    continue;
                stateForPlacement = stateForPlacement.method_11657(field_11177, direction.method_10153());
            }
        }

        return withWater(stateForPlacement, ctx);
    }

    protected boolean prefersConnectionTo(class_4538 reader, class_2338 pos, class_2350 facing) {
        class_2338 offset = pos.method_10093(facing);
        class_2680 blockState = reader.method_8320(offset);
        return FluidPipeBlock.canConnectTo(reader, offset, blockState, facing);
    }

    @Override
    public void method_66388(class_2680 state, class_3218 world, class_2338 pos, boolean isMoving) {
        if (!world.method_8608())
            FluidPropagator.propagateChangedPipe(world, pos, state);
    }

    @Override
    public boolean method_9558(class_2680 p_196260_1_, class_4538 p_196260_2_, class_2338 p_196260_3_) {
        return true;
    }

    @Override
    public void method_9615(class_2680 state, class_1937 world, class_2338 pos, class_2680 oldState, boolean isMoving) {
        if (world.method_8608())
            return;
        if (state != oldState)
            world.method_64311(pos, this, 1, class_1953.field_9310);
    }

    @Override
    public void neighborUpdate(class_2680 state, class_1937 world, class_2338 pos, class_2248 otherBlock, class_2338 neighborPos, boolean isMoving) {
        class_2350 d = FluidPropagator.validateNeighbourChange(state, world, pos, otherBlock, neighborPos, isMoving);
        if (d == null)
            return;
        if (!isOpenAt(state, d))
            return;
        world.method_64311(pos, this, 1, class_1953.field_9310);
    }

    @Override
    public void method_9612(
        class_2680 state,
        class_1937 world,
        class_2338 pos,
        class_2248 otherBlock,
        @Nullable class_9904 wireOrientation,
        boolean isMoving
    ) {
    }

    public static boolean isOpenAt(class_2680 state, class_2350 d) {
        return d.method_10166() == getPipeAxis(state);
    }

    @Override
    public void method_9588(class_2680 state, class_3218 world, class_2338 pos, class_5819 r) {
        FluidPropagator.propagateChangedPipe(world, pos, state);
    }

    protected static class_2351 getPipeAxis(class_2680 state) {
        return state.method_11654(field_11007) == class_2738.field_12471 ? class_2351.field_11052 : state.method_11654(field_11177).method_10166();
    }

    @Override
    public class_265 method_9530(class_2680 state, class_1922 p_220053_2_, class_2338 p_220053_3_, class_3726 p_220053_4_) {
        class_2738 face = state.method_11654(field_11007);
        VoxelShaper shape = face == class_2738.field_12475 ? AllShapes.SMART_FLUID_PIPE_FLOOR : face == class_2738.field_12473 ? AllShapes.SMART_FLUID_PIPE_CEILING : AllShapes.SMART_FLUID_PIPE_WALL;
        return shape.get(state.method_11654(field_11177));
    }

    @Override
    public void method_9567(class_1937 pLevel, class_2338 pPos, class_2680 pState, class_1309 pPlacer, class_1799 pStack) {
        super.method_9567(pLevel, pPos, pState, pPlacer, pStack);
        AdvancementBehaviour.setPlacedBy(pLevel, pPos, pPlacer);
    }

    @Override
    public class_2351 getAxis(class_2680 state) {
        return getPipeAxis(state);
    }

    @Override
    protected boolean method_9516(class_2680 state, class_10 pathComputationType) {
        return false;
    }

    @Override
    public class_2680 method_9559(
        class_2680 pState,
        class_4538 pLevel,
        class_10225 tickView,
        class_2338 pCurrentPos,
        class_2350 pFacing,
        class_2338 pFacingPos,
        class_2680 pFacingState,
        class_5819 random
    ) {
        updateWater(pLevel, tickView, pState, pCurrentPos);
        return pState;
    }

    @Override
    public class_3610 method_9545(class_2680 pState) {
        return fluidState(pState);
    }

    @Override
    public Class<SmartFluidPipeBlockEntity> getBlockEntityClass() {
        return SmartFluidPipeBlockEntity.class;
    }

    @Override
    public class_2591<? extends SmartFluidPipeBlockEntity> getBlockEntityType() {
        return AllBlockEntityTypes.SMART_FLUID_PIPE;
    }

    @Override
    protected @NotNull MapCodec<? extends class_2341> method_53969() {
        return field_46280;
    }

}
